/*
  Project: 9999 seconds programmable timer
  MCU: PIC12F683 running at 4.0MHz internal clock, MCLR enabled
  Details: http://embedded-lab.com/blog/?p=5569
  Date: July 12, 2012
*/

// Define Software-SPI connections for MAX7219
#define CS_Pin       GP0_bit
#define MOSI_Pin     GP1_bit
#define CLK_Pin      GP5_bit

unsigned short i, k, unit=0, ten=0;
unsigned int counter = 0, hundred=0, thousand=0, l;

void SPI_Write_Byte(unsigned short num){
 unsigned short t, Mask, Flag;
 CLK_Pin = 0;
 Mask = 128;
 for (t=0; t<8; t++){
  Flag = num & Mask;
  if(Flag == 0) MOSI_Pin = 0;
  else MOSI_Pin = 1;
  CLK_Pin = 1;
  CLK_Pin = 0;
  Mask = Mask >> 1;
 }
}

void MAX7219_INIT() {
  // Disable Shutdown mode
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x0C);    // Select Shutdown register
  SPI_Write_Byte(0x01);    // Set D0 bit to return to normal operation
  CS_Pin = 1;              // CS pin is pulled HIGH

  // Set BCD decode mode for digits DIG0-DIG3
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x09);    // Select Decode Mode register
  SPI_Write_Byte(0x0F);    // Select BCD mode for digits DIG0-DIG3
  CS_Pin = 1;              // CS pin is pulled HIGH

  // Set display brighness
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x0A);    // Select Intensity register
  SPI_Write_Byte(0x0F);    // Set maximum brightness
  CS_Pin = 1;              // CS pin is pulled HIGH

   // Set display refresh
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x0B);    // Select Scan-Limit register
  SPI_Write_Byte(0x03);    // Select digits DIG0-DIG3
  CS_Pin = 1;              // CS pin is pulled HIGH

 // Enable Display-Test
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x0F);    // Select Display-Test register
  SPI_Write_Byte(0x01);    // Enable Display-Test
  CS_Pin = 1;              // CS pin is pulled HIGH

  Delay_ms(1000);
 // Disable Display-Test
  CS_Pin = 0;              // CS pin is pulled LOW
  SPI_Write_Byte(0x0F);    // Select Display-Test register
  SPI_Write_Byte(0x00);    // Disable Display-Test
  CS_Pin = 1;              // CS pin is pulled HIGH

}

 void Display_Counter(unsigned int j){
  CS_Pin = 0;                    // CS pin is pulled LOW
  SPI_Write_Byte(4);             // Send thousands digit
  SPI_Write_Byte((j/1000)%10);
  CS_Pin = 1;                    // CS pin is pulled HIGH

  CS_Pin = 0;                    // CS pin is pulled LOW
  SPI_Write_Byte(3);             // Send hundreds digit
  SPI_Write_Byte((j/100)%10);
  CS_Pin = 1;                    // CS pin is pulled HIGH

  CS_Pin = 0;                    // CS pin is pulled LOW
  SPI_Write_Byte(2);             // Send tens digit
  SPI_Write_Byte((j/10)%10);
  CS_Pin = 1;                    // CS pin is pulled HIGH

  CS_Pin = 0;                    // CS pin is pulled LOW
  SPI_Write_Byte(1);             // Send ones digit
  SPI_Write_Byte(j%10);
  CS_Pin = 1;                    // CS pin is pulled HIGH
 }

 void update_counter(){
   counter = thousand*1000 + hundred*100 + ten*10 + unit;
 }

 void digit_refresh(unsigned short n){
   // Set display refresh
   CS_Pin = 0;              // CS pin is pulled LOW
   SPI_Write_Byte(0x0B);    // Select Scan-Limit register
   SPI_Write_Byte(n);    // Select digits DIG0-DIG3
   CS_Pin = 1;              // CS pin is pulled HIGH
 }

 void delay_debounce(){
   Delay_ms(300);
 }

 void blink_zeros(){
  unsigned short loop;
  for (loop = 0; loop < 5; loop++) {
    // Enable Shutdown mode
    CS_Pin = 0;              // CS pin is pulled LOW
    SPI_Write_Byte(0x0C);    // Select Shutdown register
    SPI_Write_Byte(0x00);    // Set D0 bit to return to normal operation
    CS_Pin = 1;              // CS pin is pulled HIGH
    delay_debounce();
    digit_refresh(3);
    // Disable Shutdown mode
    CS_Pin = 0;              // CS pin is pulled LOW
    SPI_Write_Byte(0x0C);    // Select Shutdown register
    SPI_Write_Byte(0x01);    // Set D0 bit to return to normal operation
    CS_Pin = 1;              // CS pin is pulled HIGH
    delay_debounce();
  }
 }

 unsigned short time_set_mode = 0, select_digit, TMRON = 0;
 void main() {
  TRISIO=0b00011000;     // GP3, GP4 are input
  GPIO=0x00;             // All O/Ps zero
  CMCON0 = 0x07;
  ANSEL = 0x08;            // AN3 active
  MAX7219_INIT();              // initialize  max7219
  Display_Counter(counter);
  i = 0;
  PWM1_Init(5000); // PWM module initialization (5KHz)
  PWM1_Set_Duty(127);
  do{

   if(ADC_Read(3) > 400 && ADC_Read(3) < 700 && TMRON == 0){
     l = 1000;
     while(ADC_Read(3) > 400 && ADC_Read(3) < 700){
       delay_ms(1);
       l--;
       if(l == 0 && counter > 0){
         TMRON = 1;
         PWM1_Start();
         Delay_ms(500);
         PWM1_Stop();
         time_set_mode = 0;
         break;
       }
     }
     if(!TMRON){
       digit_refresh(i);
       select_digit = i;
       i ++;
       if (i >=4) i = 0;
       time_set_mode = 1;
     }
   }

   if(ADC_Read(3) < 100 && time_set_mode == 1){
     delay_debounce();
     switch(select_digit){
       case 0: unit ++;
               if (unit > 9) unit = 0;
               break;
       case 1: ten ++;
               if (ten > 9) ten = 0;
               break;
       case 2: hundred ++;
               if (hundred > 9) hundred = 0;
               break;
       case 3: thousand ++;
               if (thousand > 9) thousand = 0;
               break;
     }
   }

   if(TMRON && counter > 0){
    counter --;
    Display_Counter(counter);
    Delay_ms(1000);
    // Initialize all relevant variables after timer runs out
    if(counter == 0) {
      TMRON = 0;
      unit = 0;
      ten = 0;
      hundred = 0;
      thousand = 0;
      i = 0;
      PWM1_Start();
      blink_zeros();
      PWM1_Stop();
    }
   }
   if(!TMRON){
     update_counter();
     Display_Counter(counter);
   }
  }while(1);
}